@@ -1,37 +1,85 @@
-/* ============================================================
- *
- * This file is part of the KDE project
- *
- * Date        : 2007-09-13
- * Description : Sane interface for KDE
- *
+/*
  * SPDX-FileCopyrightText: 2007-2011 Kare Sars <kare.sars@iki .fi>
  *
  * SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
- *
- * ============================================================ */
+ */
 
 // Local includes
 #include "labeledfslider.h"
 
+//KDE includes
+
+#include <KLocalizedString>
+
 #define FLOAT_MULTIP 32768.0
-#define TO_FLOAT(v) ((float)v / FLOAT_MULTIP)
-#define TO_FIX(v) ((int)(v * FLOAT_MULTIP))
+#define TO_DOUBLE(v) (static_cast<double>(v) / FLOAT_MULTIP)
+#define TO_FIX(v) (static_cast<int>(v * FLOAT_MULTIP))
 
 namespace KSaneIface
 {
 
 LabeledFSlider::LabeledFSlider(QWidget *parent, const QString &ltext,
-                               float min, float max, float step)
+                               double min, double max, double step)
     : KSaneOptionWidget(parent, ltext)
 {
-    int imin = TO_FIX(min);
-    int imax = TO_FIX(max);
-    m_istep = TO_FIX(step);
-    m_fstep = step;
+    initFSlider(min, max, step);
+}
+
+LabeledFSlider::LabeledFSlider(QWidget *parent, KSaneCore::Option *option)
+    : KSaneOptionWidget(parent, option)
+{
+    double maxValue = option->maximumValue().toDouble();
+    double minValue = option->minimumValue().toDouble();
+    double stepValue = option->stepValue().toDouble();
+    initFSlider(minValue, maxValue , stepValue);
+
+    QString unitSuffix;
+    KSaneCore::Option::OptionUnit unit = option->valueUnit();
+    switch (unit) {
+    case KSaneCore::Option::UnitPixel:
+        unitSuffix = i18nc("Double numbers. SpinBox parameter unit", " Pixels");
+        break;
+    case KSaneCore::Option::UnitBit:
+        unitSuffix = i18nc("Double numbers. SpinBox parameter unit", " Bits");
+        break;
+    case KSaneCore::Option::UnitMilliMeter:
+        unitSuffix = i18nc("Double numbers. SpinBox parameter unit (Millimeter)", " mm");
+        break;
+    case KSaneCore::Option::UnitDPI:
+        unitSuffix = i18nc("Double numbers. SpinBox parameter unit (Dots Per Inch)", " DPI");
+        break;
+    case KSaneCore::Option::UnitPercent:
+        unitSuffix = i18nc("Double numbers. SpinBox parameter unit (Percentage)", " %");
+        break;
+    case KSaneCore::Option::UnitMicroSecond:
+        unitSuffix = i18nc("Double numbers. SpinBox parameter unit (Microseconds)", " µs");
+        break;
+    case KSaneCore::Option::UnitSecond:
+        unitSuffix = i18nc("SpinBox parameter unit (seconds), float", " s");
+        break;
+    default:
+        unitSuffix = QString();
+        break;
+    }
+    setSuffix(unitSuffix);
+
+    setLabelText(option->title());
+    setToolTip(option->description());
+    connect(this, &LabeledFSlider::valueChanged, option, &KSaneCore::Option::setValue);
+    connect(option, &KSaneCore::Option::valueChanged, this, &LabeledFSlider::setValue);
+    double value = option->value().toDouble();
+    setValue(value);
+}
+
+void LabeledFSlider::initFSlider(double minValue, double maxValue, double stepValue)
+{
+    int imin = TO_FIX(minValue);
+    int imax = TO_FIX(maxValue);
+    m_istep = TO_FIX(stepValue);
+    m_fstep = stepValue;
     if (m_istep == 0) {
         m_istep = 1;
-        m_fstep = TO_FLOAT(m_istep);
+        m_fstep = TO_DOUBLE(m_istep);
     }
 
     //std::cout << "min=" << min << ", max=" << max << ", m_fstep="<<m_fstep<<std::endl;
@@ -44,11 +92,11 @@ LabeledFSlider::LabeledFSlider(QWidget *parent, const QString &ltext,
     m_slider->setValue(imin);
 
     m_spinb = new QDoubleSpinBox(this);
-    m_spinb->setMinimum(min);
-    m_spinb->setMaximum(max);
+    m_spinb->setMinimum(minValue);
+    m_spinb->setMaximum(maxValue);
     m_spinb->setSingleStep(m_fstep);
     int decimals = 0;
-    float tmp_step = m_fstep;
+    double tmp_step = m_fstep;
     while (tmp_step < 1) {
         tmp_step *= 10;
         decimals++;
@@ -57,17 +105,17 @@ LabeledFSlider::LabeledFSlider(QWidget *parent, const QString &ltext,
         }
     }
     m_spinb->setDecimals(decimals);
-    m_spinb->setValue(max);
+    m_spinb->setValue(maxValue);
     //m_spinb->setMinimumWidth(m_spinb->sizeHint().width()+35);
     m_spinb->setMinimumWidth(m_spinb->sizeHint().width());
     m_spinb->setAlignment(Qt::AlignRight);
-    m_spinb->setValue(min);
+    m_spinb->setValue(minValue);
 
     m_label->setBuddy(m_spinb);
 
-    connect(m_spinb,  SIGNAL(valueChanged(double)), this, SLOT(syncValues(double)));
-    connect(m_slider, SIGNAL(valueChanged(int)),    this, SLOT(syncValues(int)));
-    connect(m_slider, SIGNAL(sliderReleased()),   this, SLOT(fixValue()));
+    connect(m_spinb, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, QOverload<double>::of(&LabeledFSlider::syncValues));
+    connect(m_slider, QOverload<int>::of(&QSlider::valueChanged), this, QOverload<int>::of(&LabeledFSlider::syncValues));
+    connect(m_slider, &QSlider::sliderReleased, this, &LabeledFSlider::fixValue);
 
     m_layout->addWidget(m_slider, 0, 2);
     m_layout->addWidget(m_spinb, 0, 1);
@@ -80,12 +128,12 @@ LabeledFSlider::~LabeledFSlider()
 {
 }
 
-float LabeledFSlider::value() const
+double LabeledFSlider::value() const
 {
-    return (float)m_spinb->value();
+    return m_spinb->value();
 }
 
-float LabeledFSlider::step() const
+double LabeledFSlider::step() const
 {
     return m_fstep;
 }
@@ -95,7 +143,7 @@ void LabeledFSlider::setSuffix(const QString &text)
     m_spinb->setSuffix(text);
 }
 
-void LabeledFSlider::setRange(float min, float max)
+void LabeledFSlider::setRange(double min, double max)
 {
     //qCDebug(KSANE_LOG) << "min,max(" << m_spinb->minimum() << " - " << m_spinb->maximum();
     //qCDebug(KSANE_LOG) << ") -> (" << min << " - " << max << ")" << std::endl;
@@ -107,19 +155,19 @@ void LabeledFSlider::setRange(float min, float max)
     m_spinb->setRange(min, max);
 }
 
-void LabeledFSlider::setStep(float step)
+void LabeledFSlider::setStep(double step)
 {
     m_istep = TO_FIX(step);
     m_fstep = step;
     if (m_istep == 0) {
         m_istep = 1;
-        m_fstep = TO_FLOAT(m_istep);
+        m_fstep = TO_DOUBLE(m_istep);
     }
     m_slider->setSingleStep(m_istep);
     m_spinb->setSingleStep(m_fstep);
 
     int decimals = 0;
-    float tmp_step = m_fstep;
+    double tmp_step = m_fstep;
     while (tmp_step < 1) {
         tmp_step *= 10;
         decimals++;
@@ -130,12 +178,17 @@ void LabeledFSlider::setStep(float step)
     m_spinb->setDecimals(decimals);
 }
 
-void LabeledFSlider::setValue(float value)
+void LabeledFSlider::setValue(const QVariant &value)
 {
-    int ivalue = TO_FIX(value);
+    bool ok;
+    double newValue = value.toDouble(&ok);
+    if (!ok) {
+        return;
+    }
+    int ivalue = TO_FIX(newValue);
 
-    if (((value - m_spinb->value()) > m_fstep) || ((m_spinb->value() - value) > m_fstep)) {
-        m_spinb->setValue(value);
+    if (((newValue - m_spinb->value()) > m_fstep) || ((m_spinb->value() - newValue) > m_fstep)) {
+        m_spinb->setValue(newValue);
     } else if (ivalue != m_slider->value()) {
         m_slider->setValue(ivalue);
     }
@@ -143,7 +196,7 @@ void LabeledFSlider::setValue(float value)
 
 void LabeledFSlider::syncValues(int ivalue)
 {
-    double value = TO_FLOAT(ivalue);
+    double value = TO_DOUBLE(ivalue);
     if (((value - m_spinb->value()) > m_fstep) || ((m_spinb->value() - value) > m_fstep)) {
         m_spinb->setValue(value);
     } else if (ivalue != m_slider->value()) {
@@ -176,8 +229,10 @@ void LabeledFSlider::fixValue()
         } else {
             m_slider->setValue(m_slider->value() - rest);
         }
-        m_spinb->setValue(TO_FLOAT(m_slider->value()));
+        m_spinb->setValue(TO_DOUBLE(m_slider->value()));
     }
 }
 
 }  // NameSpace KSaneIface
+
+#include "moc_labeledfslider.cpp"
